% Blend filter example - G1600726 controls lecture 2 part III
% The script first designs prototype low pass and high pass blend filters
% for the displacement and inertial sensors respectively. These prototypes
% are the 'ideal' filters we actually want. But since we also want the sum
% of the filters to be 1, so they don't contrubute dynamics to the loop, we
% then normalize each prototype filter by the sum of the two prototypes. The
% resulting filters are the 'implemented' filters, which is what we
% ultimately use. Note, for these blends to be stable, their phase must be
% within 180 degrees of each other when their magnitudes cross.
% 
% The script makes 2 bode plots:
% 1) The prototype blends and their sum
% 2) The implemented blends and their sum
% 
% Brett Shapiro
% Last updated
% 28 March 2016

% clear all workspace variables, close all figures, and clear the command line
clear
close all
clc

%% Plot settings

% universal settings
textsize = 30;

% bode plot options
bodeopts = bodeoptions;
bodeopts.Grid = 'On';
bodeopts.FreqUnits = 'Hz';
bodeopts.MagUnits = 'abs';
bodeopts.MagScale = 'log';
bodeopts.Title.FontSize = textsize;
bodeopts.YLabel.FontSize = textsize;
bodeopts.XLabel.FontSize = textsize;
bodeopts.TickLabel.FontSize = 25;

%% Prototype filters
% for stability these must be within 180 degrees of each other where their magnitudes cross

blend_frequency = 0.1; % Hz

displacement_proto = zpk(-2*pi*blend_frequency*[0,0.1],-2*pi*blend_frequency*[1,1],1); % displacement prototype filter
inertial_proto = zpk([],-2*pi*blend_frequency*[1,10],1*10*(2*pi*blend_frequency)^2);

% sum of prototype filters
sum_proto = displacement_proto+inertial_proto;

% plot prototype filters
figure,bodeplot(displacement_proto,bodeopts),hold on
bode(inertial_proto),bode(sum_proto)

%% Implemented filters

displacement_impl = displacement_proto / sum_proto; % displacement blend filter
inertial_impl = inertial_proto / sum_proto;  % inertial blend filter

% sum of implemented filters - should equal 1!
sum_impl = displacement_impl+inertial_impl;

% plot implemented filters
figure,bodeplot(displacement_impl,bodeopts),hold on
bode(inertial_impl),bode(sum_impl)