function [fft_resp,freq] = fft_response(time_response,Ts)
% This is a supporting function for the Matlab code in the Controls Lecture
% series G1600726. 
% 
% This function calculates the FFT for a given time series (time_response) and time step (Ts in seconds). 
% The reason we don't just use the built in Matlab fft function is because it
% returns the fft without a frequency vector, and the data it does return
% has both positive and negative frequency components. Thus, this function
% calculates the frequency vector (freq) and the positive side of the fft (fft_resp).
% 
% This function is adapted from asd.m, in the seismic SVN at
% .../SeiSVN/seismic/Common/MatlabTools/
% 
% Brett Shapiro
% Last updated
% 28 March 2016

temp_fft = fft(time_response); % temporary fft; has both positive and negative frequency components

N = length(time_response); % number of data points in time series

BW = 1/(N*Ts);amp_spect_temp = sqrt(2) * Ts * temp_fft; % 'bin width', the frequency resolution of the fft

if N/2 == round(N/2)       % if there are an even number of points in the time series
	N2 = N/2 + 1;
	fft_resp         = zeros(1,N2);
	fft_resp(1)      = amp_spect_temp(1)/sqrt(2);
	fft_resp(2:N2-1) = amp_spect_temp(2:N2-1);
	fft_resp(N2)     = amp_spect_temp(N2)/sqrt(2);
else                       % if there are an odd number of points in the time series
	N2 = (N+1)/2;
	fft_resp         = zeros(1,N2);
	fft_resp(1)      = amp_spect_temp(1)/sqrt(2);
	fft_resp(2:N2)   = amp_spect_temp(2:N2);
end
freq = (0:N2-1)*BW; % frequency vector